@@ -391,12 +391,18 @@ foreach ($classes as $class => $i) {
         $out .= "{$indent}{$modifiers}function ".$f['name'];
         $out .= "(";
         $first = true;
+        $params_total = count($f['params'])-1;
         foreach ($f['params'] as $pi=>$param) {
+            $param_name = str_replace('$...', '...$', '$'.$param['name']);
+            $param_name = str_replace('"', '', $param_name);
+            if (substr($param_name, 0, 3) == '...' && $pi < $params_total) {
+                // varargs in the middle of arguments are invalid syntax.
+                // print the documentation, but ignore them in the function signature
+                continue;
+            }
             if (!$first) $out .= ", ";
             $first = false;
             if ($param['isRef']) $out .= "&";
-            $param_name = str_replace('$...', '...$', '$'.$param['name']);
-            $param_name = str_replace('"', '', $param_name);
             $out .= $param_name;
         }
         $out .= ")";
@@ -449,7 +455,7 @@ echo "wrote ".$declarationCount." declarations\n";
  * @param SplFileInfo  $file  File handler
  * @return  bool
  */
-function parseFile($file, $funcOverload="") {
+function parseFile($file, $funcOverload="", $classOverload="") {
 global $existingFunctions, $constants, $constants_comments, $variables, $classes, $isInterface, $isAbstractClass, $isFinalClass, $versions;
 
     if (substr($file->getFilename(), -4) != '.xml') return false;
@@ -520,21 +526,217 @@ global $existingFunctions, $constants, $constants_comments, $variables, $classes
         }
     }
     if (isset($xml->variablelist)) {
-        foreach ($xml->variablelist->varlistentry as $i=>$varlistentry) {
-            if ($c = (string)$varlistentry->term->constant) {
-                if (!isset($constants[$c])) {
-                    if (strpos($c, '=')) {
-                        $c = substr($c, 0, strpos($c, '='));
+        foreach ($xml->variablelist as $list) {
+            foreach ($list->varlistentry as $i=>$varlistentry) {
+                if ($c = (string)$varlistentry->term->constant) {
+                    if (!isset($constants[$c])) {
+                        if (strpos($c, '=')) {
+                            $c = substr($c, 0, strpos($c, '='));
+                        }
+                        $ctype = $varlistentry->term->type;
+                        if (!$ctype) {
+                            $ctype = $varlistentry->term->link;
+                        }
+                        $constants[$c] = (string)$ctype;
                     }
-                    $ctype = $varlistentry->term->type;
-                    if (!$ctype) {
-                        $ctype = $varlistentry->term->link;
-                    }
-                    $constants[$c] = (string)$ctype;
                 }
             }
         }
     }
+    // handle class constants (and their global aliases) not defined as fieldsynopsis
+    $consts = $xml->xpath("db:partintro//db:varlistentry");
+    foreach ($consts as $c) {
+        foreach ($c->term as $t) {
+            if ($name = (string)$t->constant) {
+                if (!isset($constants[$name])) {
+                    if (strpos($name, '=')) {
+                        $c = substr($name, 0, strpos($name, '='));
+                    }
+                    $constants[$name] = 'mixed';
+                    $constants_comments[$name] = cleanupComment($c->listitem->para->asXML());
+                }
+            }
+        }
+    }
+    // handle constants under section
+    if ($file->getFilename() == 'constants.xml') {
+        $consts = $xml->xpath("db:section//db:varlistentry");
+        foreach ($consts as $c) {
+            if ($name = (string)$c->term->constant) {
+                if (!isset($constants[$name])) {
+                    if (strpos($name, '=')) {
+                        $c = substr($name, 0, strpos($name, '='));
+                    }
+                    $ctype = $c->term->type;
+                    if (!$ctype) {
+                        $ctype = $c->term->link;
+                    }
+                    $constants[$name] = (string)$ctype;
+                }
+            }
+        }
+    }
+    // handle constants within tables
+    if ($file->getFilename() == 'constants.xml') {
+        $consts = $xml->xpath("db:section//db:row");
+        foreach ($consts as $c) {
+            if (!$c->entry && !$c->entry[0] && !$c->entry[0]->constant) {
+                continue;
+            }
+            $name = (string)$c->entry[0]->constant;
+            switch ($name) {
+                case '':
+                    continue 2;
+                default:
+                    if ($c->entry[2]) {
+                        $constants[$name] = $c->term->type;
+                        $constants_comments[$name] = cleanupComment($c->entry[2]->asXML());
+                    } else if ($c->entry[1]) {
+                        $constants[$name] = 'mixed';
+                        $constants_comments[$name] = cleanupComment($c->entry[1]->asXML());
+                    }
+            }
+        }
+    }
+    // handle constants within para
+    if ($file->getFilename() == 'constants.xml') {
+        $consts = $xml->xpath("db:para//db:varlistentry");
+        foreach ($consts as $c) {
+            if ($name = (string)$c->term->constant) {
+                if ($name == '__default') {
+                    continue;
+                }
+                if (!isset($constants[$name])) {
+                    if (strpos($name, '=')) {
+                        $c = substr($name, 0, strpos($name, '='));
+                    }
+                    $ctype = $c->term->type;
+                    if (!$ctype) {
+                        $ctype = $c->term->link;
+                    }
+                    $constants[$name] = (string)$ctype;
+                }
+            }
+        }
+    }
+    if ($file->getFilename() == 'constants.xml') {
+        $consts = $xml->xpath("db:para//db:simpara");
+        foreach ($consts as $c) {
+            if ($name = (string)$c->constant) {
+                if ($name[0] == '"') {
+                    continue;
+                }
+                if (!isset($constants[$name])) {
+                    $constants[$name] = 'mixed';
+                }
+            }
+        }
+    }
+    if ($file->getFilename() == 'ciphers.xml') {
+        $consts = $xml->xpath("db:para//db:simpara");
+        foreach ($consts as $c) {
+            if ($name = (string)$c) {
+                $name = explode(' ', $name)[0];
+                $name = explode('(', $name)[0];
+                if (!isset($constants[$name])) {
+                    $constants[$name] = 'mixed';
+                }
+            }
+        }
+    }
+    // handle constants in tables within refsect1
+    if ($file->getFilename() != 'attr.xml') {
+        $consts = $xml->xpath("db:refsect1//db:row");
+        foreach ($consts as $c) {
+            if (!$c->entry && !$c->entry[0] && !$c->entry[0]->constant) {
+                continue;
+            }
+            $name = (string)$c->entry[0]->constant;
+            switch ($name) {
+                case '':
+                case '0':
+                    continue 2;
+                case 'ABDAY_(1-7)':
+                case 'DAY_(1-7)':
+                    for ($i=1; $i<8;$i++) {
+                        $cname = substr($name, 0, strpos($name, '_') + 1) . $i;
+                        $constants[$cname] = 'mixed';
+                        if ($c->entry[1]) {
+                            $constants_comments[$cname] = cleanupComment($c->entry[1]->asXML());
+                        }
+                    }
+                    break;
+                case 'ABMON_(1-12)':
+                case 'MON_(1-12)':
+                    for ($i=1; $i<13;$i++) {
+                        $cname = substr($name, 0, strpos($name, '_') + 1) . $i;
+                        $constants[$cname] = 'mixed';
+                        if ($c->entry[1]) {
+                            $constants_comments[$cname] = cleanupComment($c->entry[1]->asXML());
+                        }
+                    }
+                    break;
+                default:
+                    $constants[$name] = 'mixed';
+                    if ($c->entry[1]) {
+                        $constants_comments[$name] = cleanupComment($c->entry[1]->asXML());
+                    }
+            }
+        }
+    }
+    // handle constants within file-upload.xml
+    if ($file->getFilename() == 'file-upload.xml') {
+        $consts = $xml->xpath("db:sect1//db:varlistentry");
+        foreach ($consts as $c) {
+            if ($name = (string)$c->term->constant) {
+                if (!isset($constants[$name])) {
+                    if (strpos($name, '=')) {
+                        $c = substr($name, 0, strpos($name, '='));
+                    }
+                    $ctype = $c->term->type;
+                    if (!$ctype) {
+                        $ctype = $c->term->link;
+                    }
+                    $constants[$name] = (string)$ctype;
+                    $constants_comments[$name] = cleanupComment($c->listitem->para->asXML());
+                }
+            }
+        }
+    }
+    // handle constants within tokens.xml
+    if ($file->getFilename() == 'tokens.xml') {
+        $consts = $xml->xpath("db:table//db:row");
+        foreach ($consts as $c) {
+            if (!$c->entry && !$c->entry[0] && !$c->entry[0]->constant) {
+                continue;
+            }
+            $name = (string)$c->entry[0]->constant;
+            switch ($name) {
+                case '':
+                    continue 2;
+                default:
+                    $constants[$name] = 'mixed';
+            }
+        }
+    }
+    // handle url-stat constants
+    if ($file->getFilename() == 'url-stat.xml') {
+        $consts = $xml->xpath("db:refsect1//db:informaltable//db:row");
+        foreach ($consts as $c) {
+            if (!$c->entry && !$c->entry[0]) {
+                continue;
+            }
+            $name = (string)$c->entry[0];
+            switch ($name) {
+                case '':
+                case 'Flag':
+                    continue 2;
+                default:
+                    $constants[$name] = 'mixed';
+                    $constants_comments[$name] = cleanupComment($c->entry[1]->asXML());
+            }
+        }
+    }
     // handle constants.xml with different layout as those above
     if ( !isset($xml->variablelist) && $file->getFilename() == 'constants.xml' && $xml->xpath("//db:constant") ) {
         $consts = $xml->xpath("//db:entry");
@@ -553,7 +755,14 @@ global $existingFunctions, $constants, $constants_comments, $variables, $classes
                         }
                     }
                 }
-                $constants[(string)$p->constant] = (string)$p->type;
+                $name = (string)$p->constant;
+                if ($name == 'LOG_LOCAL0 ... LOG_LOCAL7') {
+                    for ($i=0; $i<8; $i++) {
+                    $constants['LOG_LOCAL' . $i] = (string)$p->type;
+                    }
+                } else {
+                    $constants[$name] = (string)$p->type;
+                }
             }
         }
     } else if (!isset($xml->variablelist) && $file->getFilename() == 'commandline.xml') {
@@ -701,34 +910,81 @@ global $existingFunctions, $constants, $constants_comments, $variables, $classes
     }
 
     if (isset($xml->refsect1->methodsynopsis)) {
-        foreach( $xml->refsect1->methodsynopsis as $synopsis ) {
-            if (isset($synopsis->methodname->replaceable)) {
-                newMethodEntry('global', $synopsis->methodname->replaceable, $funcOverload, $synopsis, $desc, $xml);
-            } else {
-                newMethodEntry('global', $synopsis->methodname, $funcOverload, $synopsis, $desc, $xml);
+        $skip = false;
+        if ($funcOverload != "") {
+            foreach( $xml->refsect1->methodsynopsis as $synopsis ) {
+                if (isset($synopsis->methodname->replaceable)) {
+                    if ($funcOverload == (string) $synopsis->methodname->replaceable) {
+                        $skip = true;
+                    }
+                } else {
+                    if ($funcOverload == (string) $synopsis->methodname) {
+                        $skip = true;
+                    }
+                }
             }
+        }
+        if (!$skip) {
+            foreach( $xml->refsect1->methodsynopsis as $synopsis ) {
+                if (isset($synopsis->methodname->replaceable)) {
+                    newMethodEntry('global', $synopsis->methodname->replaceable, $funcOverload, $synopsis, $desc, $xml, $classOverload);
+                } else {
+                    newMethodEntry('global', $synopsis->methodname, $funcOverload, $synopsis, $desc, $xml, $classOverload);
+                }
 
-            $addedSomething = true;
+                $addedSomething = true;
+            }
         }
     }
     if (isset($xml->refsect1->classsynopsis) && isset($xml->refsect1->classsynopsis->methodsynopsis)) {
         $methodsynopsis = $xml->refsect1->classsynopsis->methodsynopsis;
+        $classOverloadName = $classOverload ?? $xml->refsect1->classsynopsis->ooclass->classname;
         if (isset($synopsis->methodname->replaceable)) {
-            newMethodEntry($xml->refsect1->classsynopsis->ooclass->classname, $methodsynopsis->methodname->replaceable, $funcOverload, $methodsynopsis, $desc, $xml);
+            newMethodEntry($classOverloadName, $methodsynopsis->methodname->replaceable, $funcOverload, $methodsynopsis, $desc, $xml, $classOverload);
         } else {
-            newMethodEntry($xml->refsect1->classsynopsis->ooclass->classname, $methodsynopsis->methodname, $funcOverload, $methodsynopsis, $desc, $xml);
+            newMethodEntry($classOverloadName, $methodsynopsis->methodname, $funcOverload, $methodsynopsis, $desc, $xml, $classOverload);
         }
         $addedSomething = true;
     }
-    if ( !$addedSomething && isset($xml->refnamediv->refpurpose->function) ) {
+    if ( !$addedSomething && (isset($xml->refnamediv->refpurpose->function) || isset($xml->refnamediv->refpurpose->methodname)) ) {
         // This is function alias
-        $functionName = (string)$xml->refnamediv->refname;
-        $aliasName    = (string)$xml->refnamediv->refpurpose->function;
-        $baseFileName = dirname($file->getPathname()).'/'.str_replace('_', '-', $aliasName).'.xml';
+        $functionNames[] = (string)$xml->refnamediv->refname;
+        if ($xml->refnamediv->refpurpose->function) {
+            $aliasName    = (string)$xml->refnamediv->refpurpose->function;
+            switch ($aliasName) {
+                case 'mysqli_stmt_execute':
+                    $baseFileName = dirname($file->getPathname()).'/../mysqli_stmt/execute.xml';
+                    $overload = "global";
+                    break;
+                case 'mysqli_options':
+                    $functionNames[] = 'mysqli_set_opt';
+                    $baseFileName = dirname($file->getPathname()).'/../mysqli/options.xml';
+                    $overload = "global";
+                    break;
+                case 'mysqli_real_escape_string':
+                    $baseFileName = dirname($file->getPathname()).'/../mysqli/real-escape-string.xml';
+                    $overload = "global";
+                    break;
+                case 'sql_regcase':
+                    $baseFileName = dirname($file->getPathname()).'/../../regex/functions/sql-regcase.xml';
+                    $overload = "global";
+                    break;
+                default:
+                    $baseFileName = dirname($file->getPathname()).'/'.str_replace('_', '-', $aliasName).'.xml';
+                    $overload = "";
+            }
+        } else {
+            $aliasName    = (string)$xml->refnamediv->refpurpose->methodname;
+            $baseFileName = dirname($file->getPathname()).'/'.strtolower(str_replace('::', '/', $aliasName)).'.xml';
+            $baseFileName = str_replace('/functions/', '/', $baseFileName);
+            $overload = "global";
+        }
         if ( $baseFileName == $file->getPathname() || !file_exists($baseFileName) ) {
             return false;
         }
-        parseFile(new SplFileInfo($baseFileName), $functionName);
+        foreach ($functionNames as $functionName) {
+            parseFile(new SplFileInfo($baseFileName), $functionName, $overload);
+        }
         $addedSomething = true;
     }
 
@@ -864,17 +1120,28 @@ function newPropertyEntry($class, $name, $desc, $type, $modifiers = []) {
 /**
  * create a new method entry for @p $class
  */
-function newMethodEntry($class, $function, $funcOverload, $methodsynopsis, $desc, SimpleXMLElement $xml) {
+function newMethodEntry($class, $function, $funcOverload, $methodsynopsis, $desc, SimpleXMLElement $xml, $classOverload = "") {
     global $existingFunctions, $classes;
     $class = (string) $class;
     $function = (string) $function;
     $funcOverload = (string) $funcOverload;
 
     if (strpos($function, '::')) {
-        $class = substr($function, 0, strpos($function, '::'));
+        if ($classOverload == '') {
+            $class = substr($function, 0, strpos($function, '::'));
+        }
         $function = substr($function, strpos($function, '::')+2);
+        if (strpos($funcOverload, '::')) {
+            $class = substr($funcOverload, 0, strpos($funcOverload, '::'));
+            $funcOverload = substr($funcOverload, strpos($funcOverload, '::')+2);
+        }
+    } else if (strpos($funcOverload, '::')) {
+        $class = substr($funcOverload, 0, strpos($funcOverload, '::'));
+        $funcOverload = substr($funcOverload, strpos($funcOverload, '::')+2);
     } else if (strpos($function, '->')) {
-        $class = substr($function, 0, strpos($function, '->'));
+        if ($classOverload == '') {
+            $class = substr($function, 0, strpos($function, '->'));
+        }
         $function = substr($function, strpos($function, '->')+2);
     } else {
         if ($function == '__halt_compiler') return false;
@@ -903,7 +1170,10 @@ function newMethodEntry($class, $function, $funcOverload, $methodsynopsis, $desc
     $params = array();
     foreach ($methodsynopsis->methodparam as $param) {
         $paramName = $param->parameter;
-        if (trim($paramName) == '...') continue;
+        if (trim($paramName) == '...') {
+            // Add a variable name for functions taking variable arguments
+            $paramName = '...$vararg';
+        }
         if (!trim($paramName)) continue;
         $paramName = str_replace('/', '', $paramName);
         $paramName = str_replace('-', '', $paramName);
@@ -914,7 +1184,7 @@ function newMethodEntry($class, $function, $funcOverload, $methodsynopsis, $desc
         $params[] = array(
             'name' => $paramName,
             'type' => (string)$param->type,
-            'isRef' => isset($param->parameter->attributes()->role) ? ($param->parameter->attributes()->role == "reference") : false
+            'isRef' => isset($param->parameter->attributes()->role) ? ($param->parameter->attributes()->role == "reference") : false,
         );
     }
     // get description of params
@@ -922,7 +1192,6 @@ function newMethodEntry($class, $function, $funcOverload, $methodsynopsis, $desc
         $i = 0;
         foreach ( $param_descs as $d ) {
             if ( !isset($params[$i]) ) {
-                ///TODO: support optional params (i.e. ... token)
                 continue;
             }
             $paramName = (string) $d->term->parameter;
