@@ -74,7 +74,7 @@ void DeclarationBuilder::getVariableIdentifier(VariableAst* node,
 {
     parent = QualifiedIdentifier();
     if ( node->variablePropertiesSequence ) {
-        // at least one "->" in the assigment target
+        // at least one "->" in the assignment target
         // => find he parent of the target
         // => find the target (last object property)
         if ( node->variablePropertiesSequence->count() == 1 ) {
@@ -397,6 +397,26 @@ void DeclarationBuilder::visitClassStatement(ClassStatementAst *node)
         DeclarationBuilderBase::visitClassStatement(node);
 
         importTraitMethods(node);
+    } else if (node->constsSequence) {
+        if (node->modifiers) {
+            m_currentModifers = node->modifiers->modifiers;
+            if (m_reportErrors) {
+                // have to report the errors here to get a good problem range
+                if (m_currentModifers & ModifierFinal) {
+                    reportError(i18n("Cannot use 'final' as constant modifier"), node->modifiers);
+                }
+                if (m_currentModifers & ModifierStatic) {
+                    reportError(i18n("Cannot use 'static' as constant modifier"), node->modifiers);
+                }
+                if (m_currentModifers & ModifierAbstract) {
+                    reportError(i18n("Cannot use 'abstract' as constant modifier"), node->modifiers);
+                }
+            }
+        } else {
+            m_currentModifers = 0;
+        }
+        DeclarationBuilderBase::visitClassStatement(node);
+        m_currentModifers = 0;
     } else {
         if (node->modifiers) {
             m_currentModifers = node->modifiers->modifiers;
@@ -665,7 +685,13 @@ void DeclarationBuilder::visitClassConstantDeclaration(ClassConstantDeclarationA
         }
     }
     ClassMemberDeclaration* dec = openDefinition<ClassMemberDeclaration>(identifierForNode(node->identifier), m_editor->findRange(node->identifier));
-    dec->setAccessPolicy(Declaration::Public);
+    if (m_currentModifers & ModifierProtected) {
+        dec->setAccessPolicy(Declaration::Protected);
+    } else if (m_currentModifers & ModifierPrivate) {
+        dec->setAccessPolicy(Declaration::Private);
+    } else {
+        dec->setAccessPolicy(Declaration::Public);
+    }
     dec->setStatic(true);
     dec->setKind(Declaration::Instance);
     lock.unlock();
@@ -1293,7 +1319,7 @@ void DeclarationBuilder::visitAssignmentExpressionEqual(AssignmentExpressionEqua
                 declareClassMember(ctx, type, m_findVariable.identifier, m_findVariable.node);
             }
         } else {
-            // assigment to other variables
+            // assignment to other variables
             declareVariable(currentContext(), type, m_findVariable.identifier, m_findVariable.node );
         }
     }
@@ -1400,7 +1426,7 @@ void DeclarationBuilder::visitFunctionCallParameterListElement(FunctionCallParam
         if ( refType ) {
             // this argument is referenced, so if the node contains undeclared variables we have
             // to declare them with a NULL type, see also:
-            // http://de.php.net/manual/en/language.references.whatdo.php
+            // https://de.php.net/manual/en/language.references.whatdo.php
 
             // declare with NULL type, just like PHP does
             declareFoundVariable(AbstractType::Ptr(new IntegralType(IntegralType::TypeNull)));
@@ -1599,9 +1625,27 @@ void DeclarationBuilder::closeNamespace(NamespaceDeclarationStatementAst* parent
     closeDeclaration();
 }
 
+void DeclarationBuilder::visitUseStatement(UseStatementAst* node)
+{
+    if ( node->useFunction != -1 )
+    {
+        m_useNamespaceType = FunctionDeclarationType;
+    }
+    else if ( node->useConst != -1 )
+    {
+        m_useNamespaceType = ConstantDeclarationType;
+    }
+    else
+    {
+        m_useNamespaceType = ClassDeclarationType;
+    }
+    DeclarationBuilderBase::visitUseStatement(node);
+}
+
 void DeclarationBuilder::visitUseNamespace(UseNamespaceAst* node)
 {
     DUChainWriteLocker lock;
+    bool isConstIdentifier = ( m_useNamespaceType == ConstantDeclarationType );
 
     if ( currentContext()->type() != DUContext::Namespace &&
             !node->aliasIdentifier && node->identifier->namespaceNameSequence->count() == 1 ) {
@@ -1611,23 +1655,22 @@ void DeclarationBuilder::visitUseNamespace(UseNamespaceAst* node)
         return;
     }
     IdentifierAst* idNode = node->aliasIdentifier ? node->aliasIdentifier : node->identifier->namespaceNameSequence->back()->element;
-    IdentifierPair id = identifierPairForNode(idNode);
+    IdentifierPair id = identifierPairForNode(idNode, isConstIdentifier);
 
     ///TODO: case insensitive!
-    QualifiedIdentifier qid = identifierForNamespace(node->identifier, m_editor);
-
-    DeclarationPointer dec = findDeclarationImport(ClassDeclarationType, qid);
+    QualifiedIdentifier qid = identifierForNamespace(node->identifier, m_editor, isConstIdentifier);
 
+    DeclarationPointer dec = findDeclarationImport(m_useNamespaceType, qid);
     if (!dec && !qid.explicitlyGlobal()) {
         QualifiedIdentifier globalQid = qid;
         globalQid.setExplicitlyGlobal(true);
-        dec = findDeclarationImport(ClassDeclarationType, globalQid);
+        dec = findDeclarationImport(m_useNamespaceType, globalQid);
     }
 
     if (dec)
     {
         // Check for a name conflict
-        DeclarationPointer dec2 = findDeclarationImport(ClassDeclarationType, id.second);
+        DeclarationPointer dec2 = findDeclarationImport(m_useNamespaceType, id.second);
 
         if (dec2 && dec2->context()->scopeIdentifier() == currentContext()->scopeIdentifier() &&
             dec2->context()->topContext() == currentContext()->topContext() &&
