@@ -112,9 +112,11 @@ private:
 
     bool m_gotTypeFromDocComment;
     bool m_gotReturnTypeFromDocComment;
+    bool m_gotTypeFromTypeHint;
 
     KDevelop::AbstractType::Ptr injectParseType(QString type, AstNode* node);
     KDevelop::AbstractType::Ptr parseType(QString type, AstNode* node);
+    KDevelop::AbstractType::Ptr parseSimpleType(QString type, AstNode* node);
     KDevelop::AbstractType::Ptr parseDocComment(AstNode* node, const QString& docCommentName);
     QList<KDevelop::AbstractType::Ptr> parseDocCommentParams(AstNode* node);
 };
